/**
 * Sort Bookmarks
 *
 * Sorts WeBuilder bookmarks in numeric order.
 *
 * @category  WeBuilder Plugin
 * @package   Sort Bookmarks
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2015 Peter Klein
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.0
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     42   function BubbleSort(a)
 *     60   function CloseOpenDocument()
 *     79   function SortBookmarks()
 *    116   function ReOpenDocument()
 *    125   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 5
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

/**
 * Global variable
 *
 * @var currentFilename
 */
var currentFilename;

/**
 * Simple bubblesort function
 *
 * @param  array   inputArray the array to sort
 *
 * @return array
 */
function BubbleSort(a) {
	for (var i = Length(a) - 2; i >= 0;  i--) {
		for (var j = 0; j <= i; j++) {
			if (StrToInt(a[j+1]) < StrToInt(a[j])) {
				var t = a[j];
				a[j] = a[j+1];
				a[j+1] = t;
      }
   	}
	}
	return a;
}

/**
 * Save and Close open document
 *
 * @return void
 */
function CloseOpenDocument() {
	currentFilename = Document.FileName;

	// Current file MUST be closed before modifying memory.ini
	if (currentFilename != "") {
		if (Editor.Modified) Document.Save(currentFilename);
		Document.Close(true);
		Script.TimeOut(10,"SortBookmarks");
	}
	else {
		Alert("Can't process a untitled document.\nSave the document and run the plugin again.");
	}
}

/**
 * Sort bookmarks
 *
 * @return void
 */
function SortBookmarks() {
	var config = new TIniFile(RegexReplace(Script.Path,"plugins(?!.*plugins).*", "", true) + "settings\\" + "memory.ini"),
			SL = new TStringList,
			lineNums[0],
			counter = 0;

	config.ReadSection(currentFilename, SL);	// Get section for current file

	for (var i = 0; i < SL.Count; i++) {
		if (RegexMatch(SL[i], "^Bookmark\\d?", false) != "") {
			SetLength(lineNums, counter + 1);	// Increment array size
			lineNums[counter] = config.ReadString(currentFilename, SL[i], ""); // Collect bookmark linenumbers
			counter++;
			config.DeleteKey(currentFilename, SL[i]); // Remove bookmark
		}
	}

	if (counter > 0) {
		lineNums = BubbleSort(lineNums);	// Sort the linenumbers

		for (var j = 0; j < Length(lineNums); j++) {
			config.WriteString(currentFilename, "Bookmark" + _t(j), lineNums[j]); // Write new bookmarks
		}
	}

	// Cleanup
	delete SL;
	delete config;

	Script.TimeOut(10,"ReOpenDocument");
}

/**
 * Open document again
 *
 * @return void
 */
function ReOpenDocument() {
	Documents.OpenDocument(currentFilename);
}

/**
 * Show info when plugin is installed
 *
 * @return void
 */
function OnInstalled() {
  alert("Sort Bookmarks 1.0 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");
Script.RegisterAction("", "Sort Bookmarks", "", "CloseOpenDocument");
